@@ -52,60 +52,6 @@
 G_DEFINE_TYPE_WITH_CODE (GstVaapiDisplayX11, gst_vaapi_display_x11,
     GST_TYPE_VAAPI_DISPLAY, _do_init);
 
-static const guint g_display_types = 1U << GST_VAAPI_DISPLAY_TYPE_X11;
-
-static gboolean
-parse_display_name (const gchar * name, guint * len_ptr, guint * id_ptr,
-    guint * screen_ptr)
-{
-  gulong len, id = 0, screen = 0;
-  gchar *end;
-
-  end = strchr (name, ':');
-  len = end ? end - name : strlen (name);
-
-  if (end) {
-    id = strtoul (&end[1], &end, 10);
-    if (*end == '.')
-      screen = strtoul (&end[1], &end, 10);
-    if (*end != '\0')
-      return FALSE;
-  }
-
-  if (len_ptr)
-    *len_ptr = len;
-  if (id_ptr)
-    *id_ptr = id;
-  if (screen_ptr)
-    *screen_ptr = screen;
-  return TRUE;
-}
-
-static gint
-compare_display_name (gconstpointer a, gconstpointer b)
-{
-  const GstVaapiDisplayInfo *const info = a;
-  const gchar *const cached_name = info->display_name;
-  const gchar *const tested_name = b;
-  guint cached_name_length, cached_id;
-  guint tested_name_length, tested_id;
-
-  g_return_val_if_fail (cached_name, FALSE);
-  g_return_val_if_fail (tested_name, FALSE);
-
-  if (!parse_display_name (cached_name, &cached_name_length, &cached_id, NULL))
-    return FALSE;
-  if (!parse_display_name (tested_name, &tested_name_length, &tested_id, NULL))
-    return FALSE;
-  if (cached_name_length != tested_name_length)
-    return FALSE;
-  if (strncmp (cached_name, tested_name, cached_name_length) != 0)
-    return FALSE;
-  if (cached_id != tested_id)
-    return FALSE;
-  return TRUE;
-}
-
 static inline const gchar *
 get_default_display_name (void)
 {
@@ -203,23 +149,14 @@ gst_vaapi_display_x11_open_display (GstVaapiDisplay * base_display,
   GstVaapiDisplayX11 *const display = GST_VAAPI_DISPLAY_X11_CAST (base_display);
   GstVaapiDisplayX11Private *const priv = display->priv;
 
-  GstVaapiDisplayCache *const cache = GST_VAAPI_DISPLAY_CACHE (display);
-  const GstVaapiDisplayInfo *info;
-
   if (!set_display_name (display, name))
     return FALSE;
 
-  info = gst_vaapi_display_cache_lookup_custom (cache, compare_display_name,
-      priv->display_name, g_display_types);
-  if (info) {
-    priv->x11_display = info->native_display;
-    priv->use_foreign_display = TRUE;
-  } else {
-    priv->x11_display = XOpenDisplay (get_display_name (display));
-    if (!priv->x11_display)
-      return FALSE;
-    priv->use_foreign_display = FALSE;
-  }
+  priv->x11_display = XOpenDisplay (get_display_name (display));
+  if (!priv->x11_display)
+    return FALSE;
+  priv->use_foreign_display = FALSE;
+
   priv->x11_screen = DefaultScreen (priv->x11_display);
 
   check_extensions (display);
@@ -281,25 +218,13 @@ gst_vaapi_display_x11_get_display_info (GstVaapiDisplay * display,
 {
   GstVaapiDisplayX11Private *const priv =
       GST_VAAPI_DISPLAY_X11_PRIVATE (display);
-  GstVaapiDisplayCache *const cache = GST_VAAPI_DISPLAY_CACHE (display);
-  const GstVaapiDisplayInfo *cached_info;
 
-  /* Return any cached info even if child has its own VA display */
-  cached_info = gst_vaapi_display_cache_lookup_by_native_display (cache,
-      priv->x11_display, g_display_types);
-  if (cached_info) {
-    *info = *cached_info;
-    return TRUE;
-  }
-
-  /* Otherwise, create VA display if there is none already */
   info->native_display = priv->x11_display;
   info->display_name = priv->display_name;
   if (!info->va_display) {
     info->va_display = vaGetDisplay (priv->x11_display);
     if (!info->va_display)
       return FALSE;
-    info->display_type = GST_VAAPI_DISPLAY_TYPE_X11;
   }
   return TRUE;
 }
@@ -447,6 +372,21 @@ gst_vaapi_display_x11_new_with_display (Display * x11_display)
       GST_VAAPI_DISPLAY_INIT_FROM_NATIVE_DISPLAY, x11_display);
 }
 
+GstVaapiDisplay *
+gst_vaapi_display_x11_new_with_va_display (VADisplay va_display,
+    Display * x11_display)
+{
+  GstVaapiDisplayInfo info = {
+    .va_display = va_display,
+    .native_display = x11_display,
+  };
+
+  g_return_val_if_fail (x11_display, NULL);
+
+  return gst_vaapi_display_new (g_object_new (GST_TYPE_VAAPI_DISPLAY_X11, NULL),
+      GST_VAAPI_DISPLAY_INIT_FROM_VA_DISPLAY, &info);
+}
+
 /**
  * gst_vaapi_display_x11_get_display:
  * @display: a #GstVaapiDisplayX11
