@@ -21,8 +21,6 @@
  */
 
 #include "sysdeps.h"
-#include <va/va.h>
-#include <va/va_enc_vp9.h>
 #include <gst/base/gstbitwriter.h>
 #include <gst/codecparsers/gstvp9parser.h>
 #include "gstvaapicompat.h"
@@ -39,7 +37,9 @@
 
 /* Supported set of VA rate controls, within this implementation */
 #define SUPPORTED_RATECONTROLS                  \
-  (GST_VAAPI_RATECONTROL_MASK (CQP))
+  (GST_VAAPI_RATECONTROL_MASK (CQP) |           \
+   GST_VAAPI_RATECONTROL_MASK (CBR) |           \
+   GST_VAAPI_RATECONTROL_MASK (VBR))
 
 /* Supported set of tuning options, within this implementation */
 #define SUPPORTED_TUNE_OPTIONS \
@@ -56,6 +56,9 @@
 #define MAX_FRAME_WIDTH 4096
 #define MAX_FRAME_HEIGHT 4096
 
+/* Default CPB length (in milliseconds) */
+#define DEFAULT_CPB_LENGTH 1500
+
 typedef enum
 {
   GST_VAAPI_ENCODER_VP9_REF_PIC_MODE_0 = 0,
@@ -88,9 +91,6 @@ gst_vaapi_encoder_vp9_ref_pic_mode_type (void)
 /* --- VP9 Encoder                                                      --- */
 /* ------------------------------------------------------------------------- */
 
-#define GST_VAAPI_ENCODER_VP9_CAST(encoder) \
-    ((GstVaapiEncoderVP9 *)(encoder))
-
 struct _GstVaapiEncoderVP9
 {
   GstVaapiEncoder parent_instance;
@@ -102,8 +102,47 @@ struct _GstVaapiEncoderVP9
   guint frame_num;
   GstVaapiSurfaceProxy *ref_list[GST_VP9_REF_FRAMES];   /* reference list */
   guint ref_list_idx;           /* next free slot in ref_list */
+
+  /* Bitrate contral parameters, CPB = Coded Picture Buffer */
+  guint bitrate_bits;           /* bitrate (bits) */
+  guint cpb_length;             /* length of CPB buffer (ms) */
 };
 
+/* Estimates a good enough bitrate if none was supplied */
+static void
+ensure_bitrate (GstVaapiEncoderVP9 * encoder)
+{
+  GstVaapiEncoder *const base_encoder = GST_VAAPI_ENCODER_CAST (encoder);
+  guint bitrate;
+
+  switch (GST_VAAPI_ENCODER_RATE_CONTROL (encoder)) {
+    case GST_VAAPI_RATECONTROL_CBR:
+    case GST_VAAPI_RATECONTROL_VBR:
+      if (!base_encoder->bitrate) {
+        /* FIXME: Provide better estimation */
+        /* Using a 1/6 compression ratio */
+        /* 12 bits per pixel fro yuv420 */
+        base_encoder->bitrate =
+            (GST_VAAPI_ENCODER_WIDTH (encoder) *
+            GST_VAAPI_ENCODER_HEIGHT (encoder) * 12 / 6) *
+            GST_VAAPI_ENCODER_FPS_N (encoder) /
+            GST_VAAPI_ENCODER_FPS_D (encoder) / 1000;
+        GST_INFO ("target bitrate computed to %u kbps", base_encoder->bitrate);
+      }
+
+      bitrate = (base_encoder->bitrate * 1000);
+      if (bitrate != encoder->bitrate_bits) {
+        GST_DEBUG ("HRD bitrate: %u bits/sec", bitrate);
+        encoder->bitrate_bits = bitrate;
+      }
+
+      break;
+    default:
+      base_encoder->bitrate = 0;
+      break;
+  }
+}
+
 /* Derives the profile that suits best to the configuration */
 static GstVaapiEncoderStatus
 ensure_profile (GstVaapiEncoderVP9 * encoder)
@@ -111,6 +150,9 @@ ensure_profile (GstVaapiEncoderVP9 * encoder)
   /* Always start from "simple" profile for maximum compatibility */
   encoder->profile = GST_VAAPI_PROFILE_VP9_0;
 
+  /* Ensure bitrate if not set already */
+  ensure_bitrate (encoder);
+
   return GST_VAAPI_ENCODER_STATUS_SUCCESS;
 }
 
@@ -141,7 +183,8 @@ ensure_hw_profile (GstVaapiEncoderVP9 * encoder)
   /* ERRORS */
 error_unsupported_profile:
   {
-    GST_ERROR ("unsupported HW profile (0x%08x)", encoder->profile);
+    GST_ERROR ("unsupported HW profile %s",
+        gst_vaapi_profile_get_name (encoder->profile));
     return FALSE;
   }
 }
@@ -149,11 +192,11 @@ error_unsupported_profile:
 static GstVaapiEncoderStatus
 set_context_info (GstVaapiEncoder * base_encoder)
 {
-  GstVaapiEncoderVP9 *encoder = GST_VAAPI_ENCODER_VP9_CAST (base_encoder);
+  GstVaapiEncoderVP9 *encoder = GST_VAAPI_ENCODER_VP9 (base_encoder);
   GstVideoInfo *const vip = GST_VAAPI_ENCODER_VIDEO_INFO (encoder);
   const guint DEFAULT_SURFACES_COUNT = 2;
 
-  /*Fixme:  Maximum sizes for common headers (in bytes) */
+  /* FIXME: Maximum sizes for common headers (in bytes) */
 
   if (!ensure_hw_profile (encoder))
     return GST_VAAPI_ENCODER_STATUS_ERROR_UNSUPPORTED_PROFILE;
@@ -183,6 +226,7 @@ fill_sequence (GstVaapiEncoderVP9 * encoder, GstVaapiEncSequence * sequence)
   /* keyframe maximum interval */
   seq_param->kf_max_dist = base_encoder->keyframe_period;
   seq_param->intra_period = base_encoder->keyframe_period;
+  seq_param->bits_per_second = encoder->bitrate_bits;
 
   return TRUE;
 }
@@ -216,6 +260,40 @@ error:
   }
 }
 
+static gboolean
+ensure_control_rate_params (GstVaapiEncoderVP9 * encoder)
+{
+  if (GST_VAAPI_ENCODER_RATE_CONTROL (encoder) == GST_VAAPI_RATECONTROL_CQP)
+    return TRUE;
+
+  /* RateControl params */
+  GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).bits_per_second =
+      encoder->bitrate_bits;
+  GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).window_size = encoder->cpb_length;
+
+  /* *INDENT-OFF* */
+  /* HRD params */
+  GST_VAAPI_ENCODER_VA_HRD (encoder) = (VAEncMiscParameterHRD) {
+    .buffer_size = encoder->bitrate_bits * 2,
+    .initial_buffer_fullness = encoder->bitrate_bits,
+  };
+  /* *INDENT-ON* */
+
+  return TRUE;
+}
+
+static gboolean
+ensure_misc_params (GstVaapiEncoderVP9 * encoder, GstVaapiEncPicture * picture)
+{
+  GstVaapiEncoder *const base_encoder = GST_VAAPI_ENCODER_CAST (encoder);
+
+  if (!gst_vaapi_encoder_ensure_param_quality_level (base_encoder, picture))
+    return FALSE;
+  if (!gst_vaapi_encoder_ensure_param_control_rate (base_encoder, picture))
+    return FALSE;
+  return TRUE;
+}
+
 static void
 get_ref_indices (guint ref_pic_mode, guint ref_list_idx, guint * last_idx,
     guint * gf_idx, guint * arf_idx, guint8 * refresh_frame_flags)
@@ -350,7 +428,7 @@ static GstVaapiEncoderStatus
 gst_vaapi_encoder_vp9_encode (GstVaapiEncoder * base_encoder,
     GstVaapiEncPicture * picture, GstVaapiCodedBufferProxy * codedbuf)
 {
-  GstVaapiEncoderVP9 *const encoder = GST_VAAPI_ENCODER_VP9_CAST (base_encoder);
+  GstVaapiEncoderVP9 *const encoder = GST_VAAPI_ENCODER_VP9 (base_encoder);
   GstVaapiEncoderStatus ret = GST_VAAPI_ENCODER_STATUS_ERROR_UNKNOWN;
   GstVaapiSurfaceProxy *reconstruct = NULL;
 
@@ -360,6 +438,8 @@ gst_vaapi_encoder_vp9_encode (GstVaapiEncoder * base_encoder,
 
   if (!ensure_sequence (encoder, picture))
     goto error;
+  if (!ensure_misc_params (encoder, picture))
+    goto error;
   if (!ensure_picture (encoder, picture, codedbuf, reconstruct))
     goto error;
   if (!gst_vaapi_enc_picture_encode (picture))
@@ -389,7 +469,7 @@ static GstVaapiEncoderStatus
 gst_vaapi_encoder_vp9_reordering (GstVaapiEncoder * base_encoder,
     GstVideoCodecFrame * frame, GstVaapiEncPicture ** output)
 {
-  GstVaapiEncoderVP9 *const encoder = GST_VAAPI_ENCODER_VP9_CAST (base_encoder);
+  GstVaapiEncoderVP9 *const encoder = GST_VAAPI_ENCODER_VP9 (base_encoder);
   GstVaapiEncPicture *picture = NULL;
   GstVaapiEncoderStatus status = GST_VAAPI_ENCODER_STATUS_SUCCESS;
 
@@ -421,25 +501,27 @@ gst_vaapi_encoder_vp9_reordering (GstVaapiEncoder * base_encoder,
 static GstVaapiEncoderStatus
 gst_vaapi_encoder_vp9_reconfigure (GstVaapiEncoder * base_encoder)
 {
-  GstVaapiEncoderVP9 *const encoder = GST_VAAPI_ENCODER_VP9_CAST (base_encoder);
+  GstVaapiEncoderVP9 *const encoder = GST_VAAPI_ENCODER_VP9 (base_encoder);
   GstVaapiEncoderStatus status;
 
   status = ensure_profile (encoder);
   if (status != GST_VAAPI_ENCODER_STATUS_SUCCESS)
     return status;
 
+  ensure_control_rate_params (encoder);
   return set_context_info (base_encoder);
 }
 
 static gboolean
 gst_vaapi_encoder_vp9_init (GstVaapiEncoder * base_encoder)
 {
-  GstVaapiEncoderVP9 *const encoder = GST_VAAPI_ENCODER_VP9_CAST (base_encoder);
+  GstVaapiEncoderVP9 *const encoder = GST_VAAPI_ENCODER_VP9 (base_encoder);
 
   encoder->frame_num = 0;
   encoder->loop_filter_level = DEFAULT_LOOP_FILTER_LEVEL;
   encoder->sharpness_level = DEFAULT_SHARPNESS_LEVEL;
   encoder->yac_qi = DEFAULT_YAC_QINDEX;
+  encoder->cpb_length = DEFAULT_CPB_LENGTH;
 
   memset (encoder->ref_list, 0,
       G_N_ELEMENTS (encoder->ref_list) * sizeof (encoder->ref_list[0]));
@@ -457,7 +539,7 @@ static GstVaapiEncoderStatus
 gst_vaapi_encoder_vp9_set_property (GstVaapiEncoder * base_encoder,
     gint prop_id, const GValue * value)
 {
-  GstVaapiEncoderVP9 *const encoder = GST_VAAPI_ENCODER_VP9_CAST (base_encoder);
+  GstVaapiEncoderVP9 *const encoder = GST_VAAPI_ENCODER_VP9 (base_encoder);
 
   switch (prop_id) {
     case GST_VAAPI_ENCODER_VP9_PROP_LOOP_FILTER_LEVEL:
@@ -472,6 +554,9 @@ gst_vaapi_encoder_vp9_set_property (GstVaapiEncoder * base_encoder,
     case GST_VAAPI_ENCODER_VP9_PROP_REF_PIC_MODE:
       encoder->ref_pic_mode = g_value_get_enum (value);
       break;
+    case GST_VAAPI_ENCODER_VP9_PROP_CPB_LENGTH:
+      encoder->cpb_length = g_value_get_uint (value);
+      break;
     default:
       return GST_VAAPI_ENCODER_STATUS_ERROR_INVALID_PARAMETER;
   }
@@ -558,6 +643,19 @@ gst_vaapi_encoder_vp9_get_default_properties (void)
           GST_VAAPI_ENCODER_VP9_REF_PIC_MODE_0,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstVaapiEncoderVP9:cpb-length:
+   *
+   * The size of the Coded Picture Buffer , which means
+   * the window size in milliseconds.
+   *
+   */
+  GST_VAAPI_ENCODER_PROPERTIES_APPEND (props,
+      GST_VAAPI_ENCODER_VP9_PROP_CPB_LENGTH,
+      g_param_spec_uint ("cpb-length",
+          "CPB Length", "Length of the CPB_buffer/window_size in milliseconds",
+          1, 10000, DEFAULT_CPB_LENGTH,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   return props;
 }
