@@ -159,6 +159,9 @@ static const GstVaapiEntrypointMap gst_vaapi_entrypoints[] = {
 #endif
 #if VA_CHECK_VERSION(0,39,1)
   {GST_VAAPI_ENTRYPOINT_SLICE_ENCODE_LP, VAEntrypointEncSliceLP},
+#endif
+#if USE_H264_FEI_ENCODER
+  {GST_VAAPI_ENTRYPOINT_SLICE_ENCODE_FEI, VAEntrypointFEI},
 #endif
   {0,}
 };
