@@ -120,14 +120,20 @@ create_objects (GstVaapiTextureEGL * texture, GLuint texture_id)
 
   /* ERRORS */
 error_create_image:
-  GST_ERROR ("failed to create EGL image from 2D texture %u", texture_id);
-  return FALSE;
+  {
+    GST_ERROR ("failed to create EGL image from 2D texture %u", texture_id);
+    return FALSE;
+  }
 error_create_surface:
-  GST_ERROR ("failed to create VA surface from 2D texture %u", texture_id);
-  return FALSE;
+  {
+    GST_ERROR ("failed to create VA surface from 2D texture %u", texture_id);
+    return FALSE;
+  }
 error_create_filter:
-  GST_ERROR ("failed to create VPP filter for color conversion");
-  return FALSE;
+  {
+    GST_ERROR ("failed to create VPP filter for color conversion");
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -246,9 +252,16 @@ static gboolean
 gst_vaapi_texture_egl_create (GstVaapiTextureEGL * texture)
 {
   CreateTextureArgs args = { texture };
+  GstVaapiDisplayEGL *display =
+      GST_VAAPI_DISPLAY_EGL (GST_VAAPI_OBJECT_DISPLAY (texture));
+
+  if (GST_VAAPI_TEXTURE (texture)->is_wrapped) {
+    if (!gst_vaapi_display_egl_set_current_display (display))
+      return FALSE;
+  }
 
   egl_object_replace (&texture->egl_context,
-      GST_VAAPI_DISPLAY_EGL_CONTEXT (GST_VAAPI_OBJECT_DISPLAY (texture)));
+      GST_VAAPI_DISPLAY_EGL_CONTEXT (display));
 
   return egl_context_run (texture->egl_context,
       (EglContextRunFunc) do_create_texture, &args) && args.success;
