@@ -51,9 +51,6 @@
 # include <va/va_glx.h>
 #endif
 
-/* Set to 1 to check display cache works (shared VA display) */
-#define CHECK_DISPLAY_CACHE 1
-
 static void
 print_value (const GValue * value, const gchar * name)
 {
@@ -143,91 +140,29 @@ print_formats (GArray * formats, const gchar * name)
   }
 }
 
-typedef struct _GstVaapiDisplayProperty GstVaapiDisplayProperty;
-struct _GstVaapiDisplayProperty
-{
-  const gchar *name;
-  GValue value;
-};
-
-static void
-gst_vaapi_display_property_free (GstVaapiDisplayProperty * prop)
-{
-  if (!prop)
-    return;
-  g_value_unset (&prop->value);
-  g_slice_free (GstVaapiDisplayProperty, prop);
-}
-
-static GstVaapiDisplayProperty *
-gst_vaapi_display_property_new (const gchar * name)
-{
-  GstVaapiDisplayProperty *prop;
-
-  prop = g_slice_new0 (GstVaapiDisplayProperty);
-  if (!prop)
-    return NULL;
-  prop->name = name;
-  return prop;
-}
-
-static void
-free_property_cb (gpointer data, gpointer user_data)
-{
-  gst_vaapi_display_property_free (data);
-}
-
 static void
 dump_properties (GstVaapiDisplay * display)
 {
-  GstVaapiDisplayProperty *prop;
-  GPtrArray *properties;
-  guint i;
+  GParamSpec **properties;
+  guint i, n_properties;
 
-  static const gchar *g_properties[] = {
-    GST_VAAPI_DISPLAY_PROP_RENDER_MODE,
-    GST_VAAPI_DISPLAY_PROP_ROTATION,
-    GST_VAAPI_DISPLAY_PROP_HUE,
-    GST_VAAPI_DISPLAY_PROP_SATURATION,
-    GST_VAAPI_DISPLAY_PROP_BRIGHTNESS,
-    GST_VAAPI_DISPLAY_PROP_CONTRAST,
-    NULL
-  };
+  properties =
+      g_object_class_list_properties (G_OBJECT_GET_CLASS (display),
+      &n_properties);
 
-  properties = g_ptr_array_new ();
-  if (!properties)
-    return;
-
-  for (i = 0; g_properties[i] != NULL; i++) {
-    const gchar *const name = g_properties[i];
+  for (i = 0; i < n_properties; i++) {
+    const gchar *const name = g_param_spec_get_nick (properties[i]);
+    GValue value = G_VALUE_INIT;
 
     if (!gst_vaapi_display_has_property (display, name))
       continue;
 
-    prop = gst_vaapi_display_property_new (name);
-    if (!prop) {
-      GST_ERROR ("failed to allocate GstVaapiDisplayProperty");
-      goto end;
-    }
-
-    if (!gst_vaapi_display_get_property (display, name, &prop->value)) {
-      GST_ERROR ("failed to get property '%s'", name);
-      goto end;
-    }
-    g_ptr_array_add (properties, prop);
+    g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (properties[i]));
+    g_object_get_property (G_OBJECT (display), name, &value);
+    print_value (&value, name);
   }
 
-  g_print ("%u properties\n", properties->len);
-  for (i = 0; i < properties->len; i++) {
-    prop = g_ptr_array_index (properties, i);
-    print_value (&prop->value, prop->name);
-  }
-
-end:
-  if (properties) {
-    g_ptr_array_foreach (properties, free_property_cb, NULL);
-    g_ptr_array_free (properties, TRUE);
-  }
+  g_free (properties);
 }
 
 static void
@@ -270,9 +205,6 @@ int
 main (int argc, char *argv[])
 {
   GstVaapiDisplay *display;
-#if USE_X11
-  GstVaapiDisplay *display2;
-#endif
 #if USE_X11 || USE_WAYLAND
   guint width, height;
   guint par_n, par_d;
@@ -349,46 +281,6 @@ main (int argc, char *argv[])
     display = gst_vaapi_display_x11_new (NULL);
     if (!display)
       g_error ("could not create Gst/VA display");
-
-    if (CHECK_DISPLAY_CACHE) {
-      display2 = gst_vaapi_display_x11_new (NULL);
-
-      /* Check for the same X11 display */
-      g_assert (gst_vaapi_display_x11_get_display (GST_VAAPI_DISPLAY_X11
-              (display)) ==
-          gst_vaapi_display_x11_get_display (GST_VAAPI_DISPLAY_X11 (display2)));
-
-      /* Check for the same VA display */
-      g_assert (gst_vaapi_display_get_display (display) ==
-          gst_vaapi_display_get_display (display2));
-
-      gst_vaapi_display_unref (display2);
-
-#if USE_GLX
-      display2 = gst_vaapi_display_glx_new (NULL);
-
-      /* Check for the different X11 display */
-      /* XXX: it is also desired to cache underlying X11 displays */
-      g_assert (gst_vaapi_display_x11_get_display (GST_VAAPI_DISPLAY_X11
-              (display)) !=
-          gst_vaapi_display_x11_get_display (GST_VAAPI_DISPLAY_X11 (display2)));
-
-      /* Check for different VA display */
-      g_assert (gst_vaapi_display_get_display (display) !=
-          gst_vaapi_display_get_display (display2));
-
-      gst_vaapi_display_unref (display2);
-#endif
-    }
-
-    gst_vaapi_display_get_size (display, &width, &height);
-    g_print ("Display size: %ux%u\n", width, height);
-
-    gst_vaapi_display_get_pixel_aspect_ratio (display, &par_n, &par_d);
-    g_print ("Pixel aspect ratio: %u/%u\n", par_n, par_d);
-
-    dump_info (display);
-    gst_vaapi_display_unref (display);
   }
   g_print ("\n");
 
@@ -406,16 +298,6 @@ main (int argc, char *argv[])
     if (!display)
       g_error ("could not create Gst/VA display");
 
-    if (CHECK_DISPLAY_CACHE) {
-      display2 = gst_vaapi_display_x11_new_with_display (x11_display);
-
-      /* Check for the same VA display */
-      g_assert (gst_vaapi_display_get_display (display) ==
-          gst_vaapi_display_get_display (display2));
-
-      gst_vaapi_display_unref (display2);
-    }
-
     dump_info (display);
     gst_vaapi_display_unref (display);
     XCloseDisplay (x11_display);
@@ -458,34 +340,6 @@ main (int argc, char *argv[])
     if (!display)
       g_error ("could not create Gst/VA display");
 
-    if (CHECK_DISPLAY_CACHE) {
-      display2 = gst_vaapi_display_glx_new (NULL);
-
-      /* Check for the same X11 display */
-      g_assert (gst_vaapi_display_x11_get_display (GST_VAAPI_DISPLAY_X11
-              (display)) ==
-          gst_vaapi_display_x11_get_display (GST_VAAPI_DISPLAY_X11 (display2)));
-
-      /* Check for the same VA display */
-      g_assert (gst_vaapi_display_get_display (display) ==
-          gst_vaapi_display_get_display (display2));
-
-      gst_vaapi_display_unref (display2);
-
-      display2 = gst_vaapi_display_x11_new (NULL);
-
-      /* Check for the same X11 display */
-      g_assert (gst_vaapi_display_x11_get_display (GST_VAAPI_DISPLAY_X11
-              (display)) ==
-          gst_vaapi_display_x11_get_display (GST_VAAPI_DISPLAY_X11 (display2)));
-
-      /* Check for the same VA display */
-      g_assert (gst_vaapi_display_get_display (display) ==
-          gst_vaapi_display_get_display (display2));
-
-      gst_vaapi_display_unref (display2);
-    }
-
     gst_vaapi_display_get_size (display, &width, &height);
     g_print ("Display size: %ux%u\n", width, height);
 
